package com.ipsplm.listener;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ipsplm.entity.iot.IotDeviceRealInfo;
import com.ipsplm.utils.RedisUtils;
import com.ipsplm.websocket.*;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;
import org.springframework.data.redis.listener.PatternTopic;
import org.springframework.stereotype.Component;

/**
 * @Description Redis监听器
 * @Author FanDongqiang
 * @Date 2023/5/4 11:32
 * @Version 1.0
 */
@Component
@Slf4j
@Getter
public class RedisListener implements MessageListener {
    private final PatternTopic topic = new PatternTopic("__keyevent@0__:set");

    @Autowired
    @Lazy
    private RedisUtils redisUtils;
    @Override
    public void onMessage(Message message, byte[] pattern) {
        String msg = redisUtils.get(new String(message.getBody())).toString();
        int websocketGroupId = JSON.parseObject(msg).getInteger("websocketGroupId");
        if (websocketGroupId != 0) {
            sendInfoToWebSocketHandler(websocketGroupId, msg);
        }
    }
    private void sendInfoToWebSocketHandler(int websocketGroupId, String message) {
        switch (websocketGroupId) {
            case 1:
                WebSocketHandler1.sendInfo(message);
                break;
            case 2:
                WebSocketHandler2.sendInfo(message);
                break;
            case 3:
                WebSocketHandler3.sendInfo(message);
                break;
            case 4:
                WebSocketHandler4.sendInfo(message);
                break;
            case 5:
                WebSocketHandler5.sendInfo(message);
                break;
            case 6:
                WebSocketHandler6.sendInfo(message);
                break;
            case 7:
                WebSocketHandler7.sendInfo(message);
                break;
            case 8:
                WebSocketHandler8.sendInfo(message);
                break;
            default:
                log.warn("Invalid websocketGroupId: {}", websocketGroupId);
        }
    }
}
